//---------------------------------------------------------------------------
#include <vcl.h>
#include <stdio.h>
#include <math.h>
#include <windows.h>
#include <vector>
#include <algorithm>
using namespace std;

#include "Chart.h"
#define min(a, b)  (((a) < (b)) ? (a) : (b))

//---------------------------------------------------------------------------
#pragma package(smart_init)

//! Konstruktor - rejestruje klase okna
COpenGLView::COpenGLView(HDC hDC)
{
   dt=.1;
   bInitial = true;
   bDemoLastState = false;

   this->hDC = hDC;
   //! Ustawiamy dla tego urzadzenia format pikseli
   SetDCPixelFormat(hDC);
   //! Tworzymy kontekst WinGL
   hRC = wglCreateContext(hDC);
   //! Ustawiamy jako aktualny
   wglMakeCurrent(hDC, hRC);
   //! Tworzymy czcionke
   CreateFonts(hDC);
}


//--------------------------------------------------------------------------
//! Destruktor - usuwa zarejestrowana klase
COpenGLView::~COpenGLView()
{
   //! Usuniecie kontekstu urzadzenia
   wglMakeCurrent(hDC,NULL);
   wglDeleteContext(hRC);
   DeleteFonts(hDC);
}

//--------------------------------------------------------------------------
//! Ustawienie formatu pikseli
void COpenGLView::SetDCPixelFormat(HDC hDC)
{
   int nPixelFormat;

   static PIXELFORMATDESCRIPTOR pfd = {
      sizeof(PIXELFORMATDESCRIPTOR),  // Rozmiar tej struktury
      1,                              // Wersja struktury
      PFD_DRAW_TO_WINDOW |            // Rysowanie w oknie (nie na bitmapie)
      PFD_SUPPORT_OPENGL |            // Obsuga wywoa OpenGL w tym oknie
      PFD_DOUBLEBUFFER,               // Tryb podwjnego buforowania
      PFD_TYPE_RGBA,                  // Tryb kolorw RGBA
      32,                             // Chcemy 32-bitowego koloru
      0,0,0,0,0,0,                    // Nie uywane przy wybieraniu trybu
      0,0,                            // Nie uywane przy wybieraniu trybu
      0,0,0,0,0,                      // Nie uywane przy wybieraniu trybu
      32,                             // Rozmiar bufora gbokoci - 32 bity
      0,                              // Nie uywane przy wybieraniu trybu
      0,                              // Nie uywane przy wybieraniu trybu
      PFD_MAIN_PLANE,                 // Rysowanie na gwnym planie
      0,                              // Nie uywane przy wybieraniu trybu
      0,0,0 };                        // Nie uywane przy wybieraniu trybu

   //odpowiedni format dla karty graficznej
   nPixelFormat = ChoosePixelFormat(hDC, &pfd);
   //ustaw ten format
   SetPixelFormat(hDC, nPixelFormat, &pfd);
}

//--------------------------------------------------------------------------

void COpenGLView::CreateFonts(HDC hDC)
{
   HFONT hFont;
   fontListID = glGenLists(MAX_CHARS);
   hFont = CreateFont(10,       		// Our desired HEIGHT of the font
		  0,				               // The WIDTH (If we leave this zero it will pick the best width depending on the height)
		  0,				               // The angle of escapement
		  0,				               // The angle of orientation
		  FW_BOLD,	                  // The font's weight (We want it bold)
		  FALSE,		     	            // Italic - We don't want italic
		  FALSE,		     	            // Underline - We don't want it underlined
		  FALSE,		     	            // Strikeout - We don't want it strikethrough
		  ANSI_CHARSET,	     	      // This is the type of character set
		  OUT_TT_PRECIS,  	     	   // The Output Precision
		  CLIP_DEFAULT_PRECIS,	     	// The Clipping Precision
		  ANTIALIASED_QUALITY,	     	// The quality of the font - We want anitaliased fonts
		  FF_DONTCARE|DEFAULT_PITCH,	// The family and pitch of the font.  We don't care.
		  "system");   	     	      // The font name (Like "Arial", "Courier", etc...)

   hOldFont = (HFONT)SelectObject(hDC,hFont);
   wglUseFontBitmaps(hDC, 0, MAX_CHARS-1, fontListID);
}

//--------------------------------------------------------------------------

void COpenGLView::DeleteFonts(HDC hDC)
{
   glDeleteLists(fontListID, MAX_CHARS);
   SelectObject(hDC, hOldFont);
}

//--------------------------------------------------------------------------

void COpenGLView::WriteText(float x,float y,float z,char * text)
{
   glRasterPos4f(x, y, z, 1);
   //piszemy
   glPushAttrib(GL_LIST_BIT);
   glListBase(fontListID);
   glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);
   glPopAttrib();
}

//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------
//--------------------------------------------------------------------------


//--------------------------------------------------------------------------
void COpenGLView::Clear(void)
{
   //czyszczenie buforow
   //ustalamy kolor, jakim ekran ma byc czyszczony (RGBA)
   //wartosci typu float (0.0f-1.0f)
   glClearColor(0.0f, 0.0f, 0.2f, 0.0f);
   //czyscimy ekran i bufor z
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}
//--------------------------------------------------------------------------

//inicjalizacja OpenGL - prawie wszystkie parametry
//pozostawiamy jako domyslne, ale kilka z nich
//definiujemy - mozna to tez zrobic
//bezposrednio przed rysowaniem kazdej klatki
void COpenGLView::InitOpenGL(RECT rect)
{
   //parametry pracy OpenGL:
   //Wlaczamy bufor z
   glEnable(GL_DEPTH_TEST);
   //Rysujemy piksele o wspolrzednej z mniejszej lub rownej aktualnej zawartosci
   //bufora z (mozna ustawic tez GL_LESS, ale GL_LEQUAL jest lepsze do multitexturingu
   glDepthFunc(GL_LEQUAL);
   //no i umieszczenie obszaru rysowania w oknie - pokrywamy cale okno
   glViewport(rect.left , rect.top, rect.right, rect.bottom);

   ///////////////////////////////////////////// Inicjalizacja dla drugiej metody
   // Our shading model--Gouraud (smooth)
   //  glShadeModel(GL_SMOOTH);
   // Culling.
   //glCullFace(GL_BACK);
   //glFrontFace(GL_CCW);
   //glEnable(GL_CULL_FACE);
   glEnableClientState(GL_VERTEX_ARRAY);
   glEnableClientState(GL_COLOR_ARRAY);
   glVertexPointer(3/*3 components per vertex (x,y,z)*/, GL_FLOAT, sizeof(struct Vertex), vertex);
   glColorPointer(3/*3 components per vertex (r,g,b)*/, GL_FLOAT, sizeof(struct Vertex), &vertex[0].r);  //Pointer to the first color
   glPointSize(2.0);
   ///////////////////////////////////////////// Inicjalizacja dla drugiej metody

   //Macierz projekcji - ustawiamy na standardowe 3D
   glMatrixMode(GL_PROJECTION); //wybieramy macierz projekcji
   glLoadIdentity();            //ladujemy macierz jednostkowa
   glFrustum(-0.64,0.64,-0.48,0.48,1.0,15000.0); //ladujemy macierz projekcji

   //Macierz modelowania - jej aktualny stan okresla umiejscowienie
   //i orientacje kazdego rysowanego obiektu
   //kazdy punkt dodawany przez glVertex
   //jest w rzeczywistosci wektorem o 3 (a tak na prawde 4)
   //wspolrzednych i za kazdym razem gdy taki punkt definiujemy
   //jest on przemnazany przez macierz modelowania
   //dajac w rezultacie nowy punkt przeksztalcony przez
   //wszystkie operacje zawarte w tej macierzy
   //Pozniej (na etapie rysowania) ten punkt
   //jest przepuszczany jeszcze przez macierz projekcji

   glMatrixMode(GL_MODELVIEW); //wybieramy macierz modelowania
   glLoadIdentity();           //ladujemy macierz jednostkowa

	// Ustawia wskanik na now kwadryk ( NOWE )
	quadratic = gluNewQuadric();
	// Tworzy gadkie normalne ( NOWE )
	gluQuadricNormals(quadratic, GLU_SMOOTH);
	// Tworzy wsprzdne tekstury ( NOWE )
	gluQuadricTexture(quadratic, GL_TRUE);
}

//--------------------------------------------------------------------------

void COpenGLView::DrawLocalSolutionPoint(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere)
{
   GLfloat a = fabs(vLocalSolution[j] - MinX) / fUnitX;
   GLfloat b = fabs(vLocalSolution[j + 1] - MinY) / fUnitY;
   GLfloat x = (GLfloat)(a - GRIDW/2)/(GLfloat)(GRIDW/2);   //przesuwamy na skale -1 .. 1
   GLfloat y = (GLfloat)(b - GRIDH/2)/(GLfloat)(GRIDH/2);
   GLfloat z = (GLfloat)(vLocalSolution[j + 2] - fRangeZ/2)/(GLfloat)(fRangeZ/2);
   glVertex3f(x, y, z);
   if (bDrawSphere)
   {
      glPushMatrix();
      glTranslatef(x, y, z);
      gluSphere(quadratic, 0.02f, 32, 32);
      glPopMatrix();
   }
}
//--------------------------------------------------------------------------

void COpenGLView::DrawLocalSolutionPoint2D(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere)
{
   GLfloat a = fabs(vLocalSolution[j] - MinX) / fUnitX;
   GLfloat x = (GLfloat)(a - GRIDW/2)/(GLfloat)(GRIDW/2);   //przesuwamy na skale -1 .. 1
   GLfloat z = (GLfloat)(vLocalSolution[j + 1] - fRangeZ/2)/(GLfloat)(fRangeZ/2);
   glVertex3f(x, 0, z);
   if (bDrawSphere)
   {
      glPushMatrix();
      glTranslatef(x, 0, z);
      gluSphere(quadratic, 0.02f, 32, 32);
      glPopMatrix();
   }
}

//--------------------------------------------------------------------------

void COpenGLView::DrawGASolutionPoint(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere)
{
   GLfloat a = fabs(vGASolution[j] - MinX) / fUnitX;
   GLfloat b = fabs(vGASolution[j + 1] - MinY) / fUnitY;
   GLfloat x = (GLfloat)(a - GRIDW/2)/(GLfloat)(GRIDW/2);   //przesuwamy na skale -1 .. 1
   GLfloat y = (GLfloat)(b - GRIDH/2)/(GLfloat)(GRIDH/2);
   GLfloat z = (GLfloat)(vGASolution[j + 2] - fRangeZ/2)/(GLfloat)(fRangeZ/2);
   glVertex3f(x, y, z);
   if (bDrawSphere)
   {
      glPushMatrix();
      glTranslatef(x, y, z);
      gluSphere(quadratic, 0.02f, 32, 32);
      glPopMatrix();
   }
}
//--------------------------------------------------------------------------

void COpenGLView::DrawGASolutionPoint2D(int j, float fUnitX, float fUnitY, float fRangeZ, bool bDrawSphere)
{
   GLfloat a = fabs(vGASolution[j] - MinX) / fUnitX;
   GLfloat x = (GLfloat)(a - GRIDW/2)/(GLfloat)(GRIDW/2);   //przesuwamy na skale -1 .. 1
   GLfloat z = (GLfloat)(vGASolution[j + 1] - fRangeZ/2)/(GLfloat)(fRangeZ/2);
   glVertex3f(x, 0, z);
   if (bDrawSphere)
   {
      glPushMatrix();
      glTranslatef(x, 0, z);
      gluSphere(quadratic, 0.02f, 32, 32);
      glPopMatrix();
   }
}

//--------------------------------------------------------------------------

//funkcja rysujaca pojedyncza klatke animacji wykresu
void COpenGLView::Render(RECT rect, bool bDemo, float Zoom, float kx, float ky, float PosX, float PosY,
                         bool Wireframe, bool Polygon, bool Shading, bool bSolution,
                         bool Axes, bool DescAxes, bool Frames, bool bSpiral)
{
   // inicjalizacja
   InitOpenGL(rect);
   Clear();

   //czy sciany sa cieniowane?
   if (Shading)
      glShadeModel(GL_SMOOTH);
   else
      glShadeModel(GL_FLAT);

   if (bDemoLastState != bDemo)
      bInitial = true;

   if (bInitial)
      initVertices();

   if (bDemo)
   {
      if (bInitial)
      {
         initSurface();
         bInitial = false;
      }
      else
      {
         calcSurface();
      }
      adjustGrid();
      // Move back.
      glTranslatef(PosX/1000, PosY/1000, -Zoom/1000);
      // Rotate the view
      glRotatef(kx, 1.0, 0.0, 0.0);
      glRotatef(ky, 0.0, 0.0, 1.0);
      glDrawElements(GL_QUADS, 4*QUADNUM, GL_UNSIGNED_INT, quad);
      glFinish();
   }
   //////////////////////////////////////////////////////////
   //    FUNCTION
   //////////////////////////////////////////////////////////
   else
   {
      bInitial = false;
      // Move back.
      glTranslatef(PosX/1000, PosY/1000-MinZA, -Zoom/1000);
      // Rotate the view
      glRotatef(kx, 1.0, 0.0, 0.0);
      glRotatef(ky, 0.0, 0.0, 1.0);

     if (bSpiral)
     {
      if (Polygon)
      {
         glBegin(GL_POINTS);
         for (float r = 0.0; r < 1; r = r + 0.02)
            for (float j = 0; j < 6*3.14 + 1; j = j + 0.02)
            {
              float x,y,z;
              z = j / 8 - 0.5;
              x = r * cos(j) * (1 - j / 20);
              y = r * sin(j) * (1 - j / 20);
              float r,g,b;
              g = z;
              r = 1 - z;
              b = 0;
              glColor3f(r,g,b);
              glVertex3f(x, y, z);
            }
         glEnd();
      }
     }
     else
     {
      if ((MaxY == 0) && (MinY == 0) && !((MaxX == 0) && (MinX == 0)))
      {
         glBegin(GL_LINE_STRIP);
         for (int x = 0; x < GRIDW; x++)
            glVertex3f(vertex[x].x, vertex[x].y, vertex[x].z);
         glEnd();
      }
      else
      {
         if (Polygon)
            glDrawElements(GL_QUADS, 4*QUADNUM, GL_UNSIGNED_INT, quad);
         if (Wireframe)
            glDrawArrays(GL_POINTS,0,VERTEXNUM);
         if (Frames)
            glDrawElements(GL_LINES, 4*QUADNUM, GL_UNSIGNED_INT, quad);
      }
     }



    if (bSolution)
    {
      GLfloat fRangeZ = MaxZ - MinZ;
      GLfloat fUnitX = (MaxX - MinX) / GRIDW;
      GLfloat fUnitY = (MaxY - MinY) / GRIDH;
      glColor3ub(0, 0, 255);
      //////////////////////////////////////////////////////////
      //    Local SOLUTION
      //////////////////////////////////////////////////////////
     if (bSpiral)
     {
         glColor3ub(0, 0, 255);
         for (unsigned int j = 0; j < vLocalSolution.size(); j = j + 3)
         {
            glPushMatrix();
            glTranslatef(vLocalSolution[j] * cos(vLocalSolution[j + 1]) * (1 - vLocalSolution[j + 1] / 20),
                         vLocalSolution[j] * sin(vLocalSolution[j + 1]) * (1 - vLocalSolution[j + 1] / 20),
                         vLocalSolution[j + 2]);
            gluSphere(quadratic, 0.02f, 32, 32);
            glPopMatrix();
         }
         glBegin(GL_LINE_STRIP);
         for (unsigned int j = 0; j < vLocalSolution.size(); j = j + 3)
            glVertex3f(vLocalSolution[j] * cos(vLocalSolution[j + 1]) * (1 - vLocalSolution[j + 1] / 20),
                       vLocalSolution[j] * sin(vLocalSolution[j + 1]) * (1 - vLocalSolution[j + 1] / 20),
                       vLocalSolution[j + 2]);
         glEnd();
     }
     else
     {
      if (vLocalSolution.size() > 0)
      {
         if ((MaxY == 0) && (MinY == 0))
         {
            if ((MaxX == 0) && (MinX == 0))
            {
               GLfloat z;
               if (vLocalSolution.size() > 0)
                  z = (GLfloat)(vLocalSolution[0] - MaxZ/2)/(GLfloat)(MaxZ/2);
               glVertex3f(0, 0, z);
               glPushMatrix();
               glTranslatef(0, 0, z);
               gluSphere(quadratic, 0.02f, 32, 32);
               glPopMatrix();
            }
            else
            {
               for (unsigned int j = 0; j < vLocalSolution.size(); j = j + 2)
                  DrawLocalSolutionPoint2D(j, fUnitX, fUnitY, fRangeZ, true);
               glBegin(GL_LINE_STRIP);
               for (unsigned int j = 0; j < vLocalSolution.size(); j = j + 2)
                  DrawLocalSolutionPoint2D(j, fUnitX, fUnitY, fRangeZ, false);
               glEnd();
            }
         }
         else
         {
            for (unsigned int j = 0; j < vLocalSolution.size(); j = j + 3)
               DrawLocalSolutionPoint(j, fUnitX, fUnitY, fRangeZ, true);
            glBegin(GL_LINE_STRIP);
            for (unsigned int j = 0; j < vLocalSolution.size(); j = j + 3)
               DrawLocalSolutionPoint(j, fUnitX, fUnitY, fRangeZ, false);
            glEnd();
         }
      }
     }
     if (bSpiral)
     {
         glColor3ub(255, 255, 0);
         glBegin(GL_POINTS);
         for (unsigned int j = 0; j < vGASolution.size(); j = j + 3)
            glVertex3f(vGASolution[j] * cos(vGASolution[j + 1]) * (1 - vGASolution[j + 1] / 20),
                       vGASolution[j] * sin(vGASolution[j + 1]) * (1 - vGASolution[j + 1] / 20),
                       vGASolution[j + 2]);
         glEnd();
     }
     else
     {
      //////////////////////////////////////////////////////////
      //   Global SOLUTION
      //////////////////////////////////////////////////////////
      glColor3ub(255, 255, 0);
      if (vGASolution.size() > 0)
      {
         if ((MaxY == 0) && (MinY == 0))
         {
            if ((MaxX == 0) && (MinX == 0))
            {
               GLfloat z;
               for (unsigned int j = 0; j < vGASolution.size(); j++)
                  z = (GLfloat)(vGASolution[0] - MaxZ/2)/(GLfloat)(MaxZ/2);
               glVertex3f(0, 0, z);
            }
            else
            {
               glBegin(GL_POINTS);
               for (unsigned int j = 0; j < vGASolution.size(); j = j + 2)
                  DrawGASolutionPoint2D(j, fUnitX, fUnitY, fRangeZ, false);
               glEnd();
            }
         }
         else
         {
            glBegin(GL_POINTS);
            for (unsigned int j = 0; j < vGASolution.size(); j = j + 3)
               DrawGASolutionPoint(j, fUnitX, fUnitY, fRangeZ, false);
            glEnd();
         }
      }
     }
    }
      // rysujemy osie
      if (Axes && !bSpiral)
      {
         glDisable(GL_DEPTH_TEST);
         glBegin(GL_LINES);
            glColor4ub(255,200,200,255);  // x
            glVertex3f(MinXA,0,MinZA);
            glVertex3f(MaxXA,0,MinZA);
            if ((MinYA == 0) && (MaxYA == 0))
               glVertex3f(MaxXA*0.95+MaxYA*0.05, 0.03,MinZA);
            else
               glVertex3f(MaxXA*0.95+MinXA*0.05,(MaxYA-MinYA)*0.03,MinZA);
            glVertex3f(MaxXA,0,MinZA);
            if ((MinYA == 0) && (MaxYA == 0))
               glVertex3f(MaxXA*0.95+MinXA*0.05,-0.03,0);
            else
               glVertex3f(MaxXA*0.95+MinXA*0.05,-(MaxYA-MinYA)*0.03,MinZA);
            glVertex3f(MaxXA,0,MinZA);
            if ((MaxY != 0) || (MinY != 0) && !((MaxX == 0) && (MinX == 0)))
            {
               glColor4ub(200,255,200,255);  // y
               glVertex3f(0,MinYA,MinZA);
               glVertex3f(0,MaxYA,MinZA);
               glVertex3f((MaxXA-MinXA)*0.03,MaxYA*0.95+MinYA*0.05,MinZA);
               glVertex3f(0,MaxYA,MinZA);
               glVertex3f(-(MaxXA-MinXA)*0.03,MaxYA*0.95+MinYA*0.05,MinZA);
               glVertex3f(0,MaxYA,MinZA);
            }
            glColor4ub(200,200,255,255);  // z
            glVertex3f(0,0,MinZA);
            glVertex3f(0,0,MaxZA);
            if ((MinY == 0) && (MaxY == 0))
               glVertex3f((MaxXA-MinXA)*0.03, 0, MaxZA*0.95+MinZA*0.05);
            else
               glVertex3f(0,(MaxYA-MinYA)*0.03,MaxZA*0.95+MinZA*0.05);
            glVertex3f(0,0,MaxZA);
            if ((MinY == 0) && (MaxY == 0))
               glVertex3f(-(MaxXA-MinXA)*0.03, 0, MaxZA*0.95+MinZA*0.05);
            else
               glVertex3f(0,-(MaxYA-MinYA)*0.03,MaxZA*0.95+MinZA*0.05);
            glVertex3f(0,0,MaxZA);
         glEnd();
      }

      // text
      if (DescAxes && !bSpiral)
      {
         char Value[64];
        // glColor4ub(255, 255, 255, 255);
        // WriteText(0, 0, -1, "0");
         glColor4ub(255, 200, 200, 255);
         if (MinX != 0)
         {
            sprintf(Value,"X=%g",MinX);
            WriteText(MinXA,0,MinZA,Value);
         }
         if (MaxX != 0)
         {
            sprintf(Value,"X=%g",MaxX);
            WriteText(MaxXA, 0, MinZA, Value);
         }
         glColor4ub(200,255,200,255);
         if (MinY != 0)
         {
            sprintf(Value,"Y=%g",MinY);
            WriteText(0,MinYA,MinZA,Value);
         }
         if (MaxY != 0)
         {
            sprintf(Value,"Y=%g",MaxY);
            WriteText(0, MaxYA, MinZA, Value);
         }
         glColor4ub(200,200,255,255);
         if (MinZ != 0)
         {
            sprintf(Value,"Z=%2.1f",MinZ);
            WriteText(0,0,MinZA,Value);
         }
         if (MaxZ != 0)
         {
            sprintf(Value,"Z=%2.1f",MaxZ);
            WriteText(0, 0, MaxZA, Value);
         }
      }
   }
   bDemoLastState = bDemo;
   //Oproznienie buforow sterownika OpenGL przed zamiana ekranow przez SwapBuffers())
   glFinish();
}
//-------------------------------------------------------------------------

void COpenGLView::SetVertexData(float MinX, float MaxX, float MinY, float MaxY,
                                float MinZ, float MaxZ, vVecData1D vFunction,
                                std::vector <float> vLocalSol, std::vector <float> vGASol)
{
   int iSize;

   if ((MaxZ == 0) && (MinZ == 0)) MaxZ = vFunction[0] + 10;    // dla funkcji stalej

   this->MinX = MinX;
   this->MaxX = MaxX;
   this->MinY = MinY;
   this->MaxY = MaxY;
   this->MinZ = MinZ;
   this->MaxZ = MaxZ;
   this->vLocalSolution = vLocalSol;
   this->vGASolution = vGASol;


   float fRangeZ = MaxZ - MinZ;

   for(int y = 0; y < GRIDH; y++)
      for(int x = 0; x < GRIDW; x++)
      {
         int p = y * GRIDW + x;
         vertex[p].r = 0.2;
         vertex[p].g = 0.2;
         vertex[p].b = 0.0;
       }

   int p = 0;
   // Jesli wykres 2 wymiariowy
   if ((MaxY == 0) && (MinY == 0) && !((MaxX == 0) && (MinX == 0)))
   {
      for(int x = 0; x < GRIDW; x++)
      {
         vertex[p].x = (GLfloat)(x-GRIDW/2)/(GLfloat)(GRIDW/2);
         vertex[p].y = 0;
         vertex[p].z = (GLfloat)(vFunction[p]-fRangeZ/2)/(GLfloat)(fRangeZ/2);

         //vertex[p].r = (GLfloat)mod(vFunction[p] / fRangeZ, 1.5);
         //vertex[p].g = (GLfloat)mod(vFunction[p] / fRangeZ, 1.5);
            float fCol = (GLfloat)mod(vFunction[p] / fRangeZ, 1);
            float fEx = 0.0;

            if ((fCol > 0.2) && (fCol < 0.4))
               fEx = 0.3;
            if ((fCol > 0.4) && (fCol < 0.6))
               fEx = 0.5;

            vertex[p].g = fCol;
            vertex[p].r = 1 - fCol + fEx * fCol;

            if (fCol < 0.2)
              vertex[p].b = 0;

            if (fCol < 0.05)
            {
              vertex[p].b = 0.5;
              vertex[p].r = 0.6;
              vertex[p].g = 0.2;
            }
         p++;
      }
   }
   else
   {
      for(int y = 0; y < GRIDH; y++)
         for(int x = 0; x < GRIDW; x++)
         {
            vertex[p].x = (GLfloat)(x-GRIDW/2)/(GLfloat)(GRIDW/2);
            vertex[p].y = (GLfloat)(y-GRIDH/2)/(GLfloat)(GRIDH/2);
            vertex[p].z = (GLfloat)(vFunction[p]-fRangeZ/2)/(GLfloat)(fRangeZ/2);

            float fCol = (vertex[p].z + 1) / 2;
            float fG;

            vertex[p].r = 1.9*fCol;

            fG = 1.2*fCol*fCol;
            vertex[p].g = 1 - fG;
            if (fCol < 0.15)
               vertex[p].g = 1 - 0.1*pow(6,fCol);
            if ((fCol < 0.05))
               vertex[p].g = 1 - 0.5*pow(6,fCol);
            if (fCol < 0.2)
               vertex[p].b = 3*fCol*fCol;

            p++;
         }
   }

   MinXA = (-GRIDW/2)/(GRIDW/2);
   MaxXA = (GRIDW-GRIDW/2)/(GRIDW/2);
   MinYA = (-GRIDH/2)/(GRIDH/2);
   MaxYA = (GRIDH-GRIDH/2)/(GRIDH/2);
   MinZA = (MinZ-fRangeZ/2)/(fRangeZ/2);
   MaxZA = (MaxZ-fRangeZ/2)/(fRangeZ/2);
}

//------------------------------------------------------------------------------

void COpenGLView::initVertices()
{
   int x,y,p;
   // place the vertices in a grid
   for(y = 0; y < GRIDH; y++)
      for(x = 0; x < GRIDW; x++)
      {
         p = y * GRIDW + x;

         vertex[p].x = (GLfloat)(x-GRIDW/2)/(GLfloat)(GRIDW/2);
         vertex[p].y = (GLfloat)(y-GRIDH/2)/(GLfloat)(GRIDH/2);
         vertex[p].z = 0;

         vertex[p].b = 1.0;
         vertex[p].r = 0.0;
         vertex[p].g = 0.0;
       }

   for(y = 0; y < QUADH; y++)
      for(x = 0; x < QUADW; x++)
      {
         p = 4*(y*QUADW + x);

         /* first quad */
         quad[p+0] = y    *GRIDW+x;    /* some point */
         quad[p+1] = y    *GRIDW+x+1;  /* neighbor at the right side */
         quad[p+2] = (y+1)*GRIDW+x+1;  /* upper right neighbor */
         quad[p+3] = (y+1)*GRIDW+x;    /* upper neighbor */
      }
}

//------------------------------------------------------------------------------

void COpenGLView::initSurface()
{
  int x, y;
  double d2, d;

   for(y = 0; y<GRIDH; y++)
   {
      for(x = 0; x<GRIDW; x++)
      {
         d2 = pow(x - GRIDW/2, 2) + pow(y - GRIDH/2, 2);
         d = sqrt(d2);
         if(d < .1 * (GRIDW/2))
         {
   	      d = d * 10.0;
      	   p[x][y] = -cos(d * M_PI / (GRIDW/2) / 2) * 100;
         }
         else
         {
	         p[x][y]=0;
         }
         vx[x][y] = 0;
         vy[x][y] = 0;
      }
   }
}

//------------------------------------------------------------------------------

// float Modulo function
double COpenGLView::mod(double value, double modulo)
{
  return value-(modulo*floor(value/modulo));
}

//------------------------------------------------------------------------------

// Modify the height of each vertex according to the pressure
void COpenGLView::adjustGrid()
{
   int pos;
   int x, y;

   for(y = 0; y<GRIDH; y++)
      for(x = 0; x<GRIDW; x++)
      {
         pos = y*GRIDW + x;
         vertex[pos].z = p[x][y]/50.0;
         vertex[pos].b = (GLfloat)mod(p[x][y]/15.0,1.0);
      }
}

//------------------------------------------------------------------------------

void COpenGLView::calcSurface()
{
   int x, y, x2, y2;
   double d;

   // compute accelerations
   for(x = 0;x< GRIDW; x++)
   {
      x2 = (int)(mod(x + 1, GRIDW));
      for(y = 0;y<GRIDH;y++)
      {
         d = p[x][y] - p[x2][y];
         ax[x][y] = d;
      }
   }

   for(y = 0;y< GRIDH;y++)
   {
      y2 = (int)(mod(y + 1, GRIDH));
      for(x = 0; x<GRIDW; x++)
      {
         d = p[x][y] - p[x][y2];
         ay[x][y] = d;
      }
   }

   // compute speeds
   for(x = 0; x<GRIDW; x++)
   {
      for(y = 0; y<GRIDH; y++)
      {
         vx[x][y] = vx[x][y] + ax[x][y] * dt;
         vy[x][y] = vy[x][y] + ay[x][y] * dt;
      }
   }

   // compute pressure
   for(x = 1; x<GRIDW; x++)
   {
      x2 = (int)mod(x - 1, GRIDW);
      for(y = 1; y<GRIDH; y++)
      {
         y2 = (int)mod(y - 1, GRIDH);
         p[x][y] = p[x][y] + (vx[x2][y] - vx[x][y] + vy[x][y2] - vy[x][y]) * dt;
      }
   }
}

   /* //rysujemy siatke
   //wylaczamy buforowanie Z - siatka nie ma byc
   // zaslaniana przez wielokaty nawet jesli
   // znajduja sie blizej obserwatora
   glDisable(GL_DEPTH_TEST);
   //plynne sterowanie przezroczystoscia siatki
   if (Blending)
      glEnable(GL_BLEND);
   //funkcja mieszania kolorow
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   //kolor siatki (4 wartosci typu float - 0.0-1.0)
   //R=0.0 G=0.0 B=0.0
   //Alpha=0.0 - 1.0 w zaleznosci od powiekszenia wykresu
   //im dalej oddalimy tym bardziej przezroczysta staje sie siatka
   //zeby uniknac efektu zlewania sie linii siatki w calosc
   //jesli chcesz zobaczyc, jak bylo wczesniej ustaw Blending na false
   //przed wywolaniem Wykres.Execute();

  /**/

