//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "OptimizationThread.h"
#pragma package(smart_init)
//---------------------------------------------------------------------------

//   Important: Methods and properties of objects in VCL can only be
//   used in a method called using Synchronize, for example:
//
//      Synchronize(UpdateCaption);
//
//   where UpdateCaption could look like:
//
//      void __fastcall OptimizaationThread::UpdateCaption()
//      {
//        Form1->Caption = "Updated in a thread";
//      }
//---------------------------------------------------------------------------

__fastcall OptimizationThread::OptimizationThread(bool CreateSuspended,
                    int iTypeAlg, CAlgParameters AlgParam) : TThread(CreateSuspended)
{
   this->iTypeAlg = iTypeAlg;
   this->AlgParam = AlgParam;
}
//---------------------------------------------------------------------------
void __fastcall OptimizationThread::Execute()
{
   bThreadFinish = false;
   Solution.Clear();
   if (!Terminated)
   {
      switch (iTypeAlg)
      {
         case 0: Solution = ls.Greedy(AlgParam);
                 break;
         case 1: Solution = ls.Steepest(AlgParam);
                 break;
         case 2: Solution = ls.SimulatedAnnealing(AlgParam);
                 break;
         case 3: Solution = gs.GeneticAlgorithm(AlgParam);
                 break;
      }
   }
   bThreadFinish = true;
}
//---------------------------------------------------------------------------
CSolution __fastcall OptimizationThread::GetResults()
{
   if (iTypeAlg != 3)
      return ls.GetSolution();
   else
      return gs.GetSolution();
}
//---------------------------------------------------------------------------
CSolution __fastcall OptimizationThread::GetPopulationGA()
{
   CSolution pop;
   pop.Clear();
   if (iTypeAlg == 3)
      return gs.GetPopulation();
   else
      return pop;
}
//---------------------------------------------------------------------------
bool __fastcall OptimizationThread::GetState()
{
   return bThreadFinish;
}



